#
# coding=utf-8
"""test_isbn13 - Test Isbn13 class."""
# Copyright © 2012-2017  James Rowe <jnrowe@gmail.com>
#
# This file is part of pyisbn.
#
# pyisbn is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyisbn is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyisbn.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0+

from pytest import mark

from pyisbn import Isbn13

from tests.data import TEST_ISBN13S


@mark.parametrize('isbn', TEST_ISBN13S + ['978-052-187-1723', ])
def test_calculate_checksum(isbn):
    assert Isbn13(isbn).calculate_checksum() == isbn[-1]


@mark.parametrize('isbn', TEST_ISBN13S + ['9780071148160', ])
def test_convert(isbn):
    assert Isbn13(isbn).convert()[:-1] == isbn[3:-1]



#
# coding=utf-8
"""test_functions - Test function interface."""
# Copyright © 2012-2017  James Rowe <jnrowe@gmail.com>
#
# This file is part of pyisbn.
#
# pyisbn is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyisbn is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyisbn.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0+

import unicodedata

from pytest import mark, raises

from pyisbn import (IsbnError, _isbn_cleanse, calculate_checksum, convert,
                    validate)
from tests.data import TEST_ISBNS


@mark.parametrize('isbn', TEST_ISBNS)
def test__isbn_cleanse_sbn(isbn):
    if isbn.startswith('0'):
        assert _isbn_cleanse(isbn[1:]) == isbn
        assert _isbn_cleanse(isbn[1:-1], False) == isbn[:-1]


@mark.parametrize('isbn', TEST_ISBNS)
def test__isbn_cleanse_isbn(isbn):
    assert _isbn_cleanse(isbn) == isbn
    assert _isbn_cleanse(isbn[:-1], False) == isbn[:-1]


# See tests.test_regressions.test_issue_7_unistr
@mark.parametrize('isbn', [
    unicodedata.lookup('EN DASH').join(['978', '1', '84724', '253', '2']),
    unicodedata.lookup('EM DASH').join(['978', '0', '385', '08695', '0']),
    unicodedata.lookup('HORIZONTAL BAR').join(['978', '0199564095']),
])
def test__isbn_cleanse_unicode_dash(isbn):
    assert _isbn_cleanse(isbn) == "".join(filter(lambda s: s.isdigit(), isbn))


@mark.parametrize('isbn', [
    unicodedata.lookup('EN DASH').join(['978', '1', '84724', '253', '2']),
    '978-0-385-08695-0',
])
def test__isbn_cleanse_reflect_type(isbn):
    assert type(_isbn_cleanse(isbn)) == type(isbn)


def test__isbn_cleanse_invalid_type():
    with raises(TypeError, match="ISBN must be a string, received 2"):
        _isbn_cleanse(2)


@mark.parametrize('checksum,message', [
    (True, 'ISBN must be either 10 or 13 characters long'),
    (False, 'ISBN must be either 9 or 12 characters long without checksum'),
])
def test__isbn_cleanse_invalid_length(checksum, message):
    with raises(IsbnError, match=message):
        _isbn_cleanse('0-123', checksum=checksum)


@mark.parametrize('isbn,message', [
    ('0-x4343', 'non-digit parts'),
    ('012345678-b', 'non-digit or X checksum'),
    ('012345678901b', 'non-digit checksum'),
    ('xxxxxxxxxxxx1', 'non-digit parts'),
    ('0x0000000', 'non-digit parts'),
])
def test__isbn_cleanse_invalid(isbn, message):
    with raises(IsbnError, match=message):
        _isbn_cleanse(isbn)


def test__isbn_cleanse_invalid_no_checksum():
    with raises(IsbnError, match='non-digit parts'):
        _isbn_cleanse('0x0000000', False)


@mark.parametrize('isbn', TEST_ISBNS)
def test_calculate_checksum(isbn):
    assert calculate_checksum(isbn[:-1]) == isbn[-1]


@mark.parametrize('isbn', TEST_ISBNS)
def test_convert(isbn):
    assert convert(convert(isbn)) == isbn


def test_convert_invalid():
    with raises(IsbnError, match='Only ISBN-13s with 978 Bookland code can be '
                                 'converted to ISBN-10.'):
        convert('0000000000000')


@mark.parametrize('isbn', TEST_ISBNS)
def test_validate(isbn):
    assert validate(isbn)


@mark.parametrize('isbn', [
    '1-234-56789-0',
    '2-345-6789-1',
    '3-456-7890-X',
])
def test_validate_invalid(isbn):
    assert validate(isbn) is False



#
# coding=utf-8
"""data - ISBNs for use in tests."""
# Copyright © 2012-2017  James Rowe <jnrowe@gmail.com>
#
# This file is part of pyisbn.
#
# pyisbn is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyisbn is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyisbn.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0+

import json


with open('tests/books.json') as f:
    #: Sample book data for use in tests
    TEST_BOOKS = json.load(f)

#: ISBNs from sample book data for use in tests
TEST_ISBNS = [s.replace('-', '') for s in TEST_BOOKS.values()]
#: ISBN 10s from sample book data for use in tests
TEST_ISBN10S = [s for s in TEST_ISBNS if len(s) == 10]
#: ISBN 13s from sample book data for use in tests
TEST_ISBN13S = [s for s in TEST_ISBNS if len(s) == 13]



#
# coding=utf-8
"""test_isbn - Test Isbn class."""
# Copyright © 2012-2017  James Rowe <jnrowe@gmail.com>
#
# This file is part of pyisbn.
#
# pyisbn is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyisbn is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyisbn.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0+

from sys import version_info

from pytest import mark, raises

from pyisbn import (CountryError, Isbn, SiteError)
from tests.data import TEST_ISBNS


@mark.parametrize('isbn', TEST_ISBNS + [
    '9780521871723',
    '3540009787',
])
def test___repr__(isbn):
    assert repr(Isbn(isbn)) == 'Isbn(%r)' % isbn


@mark.parametrize('isbn', TEST_ISBNS + [
    '9780521871723',
    '978-052-187-1723',
    '3540009787',
])
def test___str__(isbn):
    assert str(Isbn(isbn)) == 'ISBN %s' % isbn


@mark.skipif(version_info < (2, 6),
             reason="format() not supported with this Python version")
@mark.parametrize('isbn,format_spec,result',
    [(s, '', 'ISBN %s' % s) for s in TEST_ISBNS] + [
    ('9780521871723', '', 'ISBN 9780521871723'),
    ('978-052-187-1723', 'urn', 'URN:ISBN:978-052-187-1723'),
    ('3540009787', 'url',
        'https://www.amazon.com/s?search-alias=stripbooks&field-isbn='
        '3540009787'),
    ('3540009787', 'url:amazon:uk',
        'https://www.amazon.co.uk/s?search-alias=stripbooks&field-isbn='
        '3540009787'),
    ('3540009787', 'url:amazon',
        'https://www.amazon.com/s?search-alias=stripbooks&field-isbn='
        '3540009787'),
])
def test___format__(isbn, format_spec, result):
    assert format(Isbn(isbn), format_spec) == result


def test___format__invalid_format_spec():
    with raises(ValueError, match="Unknown format_spec 'biscuit'"):
        format(Isbn('0071148167'), 'biscuit')


@mark.parametrize('isbn,result', [(s, s[-1]) for s in TEST_ISBNS] + [
    ('978-052-187-1723', '3'),
    ('3540009787', '7'),
    ('354000978', '7'),
])
def test_calculate_checksum(isbn, result):
    assert Isbn(isbn).calculate_checksum() == result


@mark.parametrize('isbn,result', [
    ('0071148167', '9780071148160'),
    ('9780071148160', '0071148167'),
])
def test_convert(isbn, result):
    assert Isbn(isbn).convert() == result


@mark.parametrize('isbn,result', [(s, True) for s in TEST_ISBNS] + [
    ('978-052-187-1723', True),
    ('978-052-187-1720', False),
    ('3540009787', True),
    ('354000978x', False),
])
def test_validate(isbn, result):
    assert Isbn(isbn).validate() == result


@mark.parametrize('country,result', [
    ('us', '.com/s?search-alias=stripbooks&field-isbn=0071148167'),
    ('uk', '.co.uk/s?search-alias=stripbooks&field-isbn=0071148167'),
    ('de', '.de/s?search-alias=stripbooks&field-isbn=0071148167'),
])
def test_to_url(country, result):
    assert Isbn('0071148167').to_url(country=country) \
        == 'https://www.amazon' + result


def test_to_url_invalid_country():
    with raises(CountryError, match="zh"):
        Isbn('0071148167').to_url(country='zh')


@mark.parametrize('site,result', [
    ('copac', 'http://copac.jisc.ac.uk/search?isn=0071148167'),
    ('google', 'https://books.google.com/books?vid=isbn:0071148167'),
    ('isbndb', 'https://isbndb.com/search/all?query=0071148167'),
    ('waterstones',
     'https://www.waterstones.com/books/search/term/0071148167'),
    ('whsmith',
     'https://www.whsmith.co.uk/search/go?w=0071148167&af=cat1:books'),
    ('worldcat', 'http://worldcat.org/isbn/0071148167'),
])
def test_to_url_site(site, result):
    assert Isbn('0071148167').to_url(site=site) == result


def test_to_url_invalid_site():
    with raises(SiteError, match="nosite"):
        Isbn('0071148167').to_url(site='nosite')


@mark.parametrize('isbn,result',
    [(s, 'URN:ISBN:%s' % s) for s in TEST_ISBNS]
)
def test_to_urn(isbn, result):
    assert Isbn(isbn).to_urn() == result



#
# coding=utf-8
"""conf - Sphinx configuration information."""
# Copyright © 2011-2018  James Rowe <jnrowe@gmail.com>
#
# This file is part of pyisbn.
#
# pyisbn is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyisbn is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyisbn.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0+

import os
import sys

from subprocess import CalledProcessError
try:
    from subprocess import check_output
except ImportError:
    from subprocess import (PIPE, Popen)

    def check_output(cmd):
        process = Popen(cmd, stdout=PIPE)
        out, _ = process.communicate()
        ret = process.wait()
        if ret:
            raise CalledProcessError(ret, cmd[0])
        return out


root_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.insert(0, root_dir)

import pyisbn  # NOQA

extensions = \
    ['sphinx.ext.%s' % ext for ext in ['autodoc', 'coverage', 'doctest',
                                       'extlinks', 'ifconfig', 'intersphinx',
                                       'napoleon', 'todo', 'viewcode']] \
    + ['sphinxcontrib.%s' % ext for ext in []]

# Only activate spelling if it is installed.  It is not required in the
# general case and we don't have the granularity to describe this in a clean
# way
try:
    from sphinxcontrib import spelling  # NOQA
except ImportError:
    pass
else:
    extensions.append('sphinxcontrib.spelling')

master_doc = 'index'
source_suffix = '.rst'

project = 'pyisbn'
try:
    unicode
    copyright = pyisbn.__copyright__.decode('utf-8')
except NameError:
    copyright = pyisbn.__copyright__

version = '.'.join(map(str, pyisbn._version.tuple[:2]))
release = pyisbn._version.dotted

rst_prolog = """
.. |ISBN| replace:: :abbr:`ISBN (International Standard Book Number)`
.. |PYPI| replace:: :abbr:`PyPI (Python Package Index)`
.. |modref| replace:: :mod:`pyisbn`
"""

pygments_style = 'sphinx'
try:
    html_last_updated_fmt = check_output(['git', 'log',
                                          "--pretty=format:'%ad [%h]'",
                                          '--date=short', '-n1'])
except CalledProcessError:
    pass

# Autodoc extension settings
autoclass_content = 'init'
autodoc_default_flags = ['members', ]

# intersphinx extension settings
intersphinx_mapping = {
    k: (v, os.getenv('SPHINX_%s_OBJECTS' % k.upper()))
    for k, v in {
        'python': 'http://docs.python.org/',
    }.items()
}

# extlinks extension settings
extlinks = {
    'pypi': ('http://pypi.python.org/pypi/%s', ''),
    'issue': ('https://github.com/JNRowe/jnrbase/issues/%s', 'GitHub #'),
}

# spelling extension settings
spelling_lang = 'en_GB'
spelling_word_list_filename = 'wordlist.txt'

# napoleon extension settings
napoleon_numpy_docstring = False



#
# coding=utf-8
"""test_regressions - Test for regressions."""
# Copyright © 2012-2017  James Rowe <jnrowe@gmail.com>
#
# This file is part of pyisbn.
#
# pyisbn is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# pyisbn is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# pyisbn.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0+

import unicodedata

from pytest import mark

from pyisbn import _isbn_cleanse


# The lookup hoop jumping here is to make it easier to generate native Unicode
# types on the various supported Python versions.
@mark.parametrize('isbn', [
    '978-1-84724-253-2',
    unicodedata.lookup('EN DASH').join(['978', '1', '84724', '253', '2']),
    unicodedata.lookup('HORIZONTAL BAR').join(['978', '0199564095']),
])
def test_issue_7_unistr(isbn):
    assert _isbn_cleanse(isbn) == "".join(filter(lambda s: s.isdigit(), isbn))
